/*= driver gx1025-1050 ====================================================*/
/* Original Release: 01/06/2017                                            */
/* By: O.L.                                                                */
/* Originally written in C                                                 */
/*=========================================================================*/


/********** HEADER FILES ************************************/
#include <userint.h>
#include <formatio.h>
#include <utility.h>
#include <ansi_c.h>
#include "gx1025-1050.h"
#include <visa.h> 


#define T_OUT 			1000
#define	USB_VID			"0xF4ED"
#define	USB_PID			"0xEE3A"
#define	NB_PTS			16384	// Number of points for arbitrary signal

// definition of modulation types
#define	OFF_MOD			0
#define	AM				1
#define	DSB_AM			2
#define ASK				3
#define	FM				4
#define	FSK				5
#define PM				6


/*= UTILITY ROUTINES ======================================================*/
int gx1025_send (char *buff);
int gx1025_invalid_integer_range (int val, int min, int max, int err_code);
int gx1025_invalid_real_range (double val, double min, double max, int err_code);



/*= STATIC VARIABLES ======================================================*/
static int gx1025_err;
static char *function[7] = {"SINE","SQUARE","RAMP","PULSE","NOISE","ARB","DC"};
static char *onOff[2] = {"OFF","ON"};
static char *funcModul[7] = {"OFF","AM","DSBAM","ASK","FM","FSK","PM"};
static char *shapeFunc[7] = {"SINE","SQUARE","TRIANGLE","UPRAMP","DNRAMP","NOISE","ARB"};
static char *srceGen[3] = {"INT","EXT","MAN"};
static char *swLaw[2] = {"LINE","LOG"};
static char *sDirec[2] = {"UP","DOWN"};
static char *typBurst[2] = {"NCYC","GATE"};
static char *polarity[2] = {"POS","NEG"};
static char *srceBurst[3] = {"OFF","RISE","FALL"};

/*= GLOBAL VARIABLES ======================================================*/
char StrIdent[100];
char identGX[200];				// serial number
ViSession sessionCom;
ViSession sessionUSB;
double FreqMax;



//----------------------------------------------------------------
// Init by USB port
//----------------------------------------------------------------
int _VI_FUNC gx1025_init (void)
{
	char buff[300], nomGX[15];
	char versionGX[30], numSerie[30];
	ViUInt32 retCnt;
	char Instrument_Descriptor[256];
	ViFindList Find_Handle=0;
	ViPUInt32 Return_Count=0;


	*buff = 0;
	gx1025_err = -1;
		
	viOpenDefaultRM (&sessionCom);
	   
	// ressources with ID vendor : 0xF4ED
	// connection with "USB::vendor id::product id::serial number::RAW"
	Fmt (buff, "USB?*INSTR{VI_ATTR_MANF_ID == %s}", USB_VID);
	if (viFindRsrc (sessionCom, buff, &Find_Handle, Return_Count, Instrument_Descriptor) == 0 &&
		viOpen (sessionCom, Instrument_Descriptor, VI_EXCLUSIVE_LOCK, VI_NULL, &sessionUSB) == 0)
	{	// Open success 
	    // Configure VISA Formatted I/O
	    viSetAttribute (sessionUSB, VI_ATTR_TERMCHAR, '\n');
	    viSetAttribute (sessionUSB, VI_ATTR_TMO_VALUE, T_OUT);
	    viSetBuf (sessionUSB, VI_READ_BUF|VI_WRITE_BUF, 5000);
	    viSetAttribute (sessionUSB, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS);
	    viSetAttribute (sessionUSB, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS);
		viSetAttribute (sessionUSB, VI_ATTR_SUPPRESS_END_EN, VI_FALSE);
		viFlush (sessionUSB, VI_WRITE_BUF);
	    viFlush (sessionUSB, VI_READ_BUF);

		gx1025_send ("\n");
		Delay (0.1);
		gx1025_send ("*IDN?\n");
		memset (StrIdent, 0, 100);
		viRead (sessionUSB, StrIdent, 100, &retCnt);
	    SetWaitCursor(0);  
		
		if (FindPattern (StrIdent, 0, -1, "GX1025", 0, 0) != -1 ||
			FindPattern (StrIdent, 0, -1, "GX1050", 0, 0) != -1)
		{
			gx1025_err = 0;
			memset (nomGX, 0, 15);
			Scan (StrIdent, "%s[dt44],%s[t44],%s[t44],%s[t44]", nomGX, numSerie, versionGX);
			Fmt (identGX, "%s - v%s - S/N:%s", nomGX, versionGX, numSerie);
			
			if (FindPattern (StrIdent, 0, -1, "GX1050", 0, 0) != -1)
				FreqMax = 50e6;		// GX1050 model : 50MHz
			else
				FreqMax = 25e6;		// GX1025 model : 25MHz
		
			Delay (0.2);
		}
		else
			gx1025_close ();
	}
	else	// Failed: session close
	{	
		viClose (sessionCom);
	}
		
	return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Waveform 
//----------------------------------------------------------------
int _VI_FUNC gx1025_ConfigMain (int channel, int func)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (func, 0, 6, -2) != 0)
        return gx1025_err;

    Fmt (cmd, "C%i:BSWV WVTP,%s\r", channel + 1, function[func]);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Frequency
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainFreq (int channel, double frequency)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (frequency, 1e-6, FreqMax, -2) != 0)
        return gx1025_err;
        
    Fmt (cmd, "C%i:BSWV FRQ,%f\r", channel + 1, frequency);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Amplitude
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainAmplitude (int channel, int wave, double amplitude)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (wave == 4)
	{
		if (gx1025_invalid_real_range (amplitude, 0.0005, 20.0, -2) != 0)
	        return gx1025_err;
	}
	else
	{
	    if (gx1025_invalid_real_range (amplitude, 0.004, 20.0, -2) != 0)
	        return gx1025_err;
	}
        
    if (wave == 4)
		Fmt (cmd, "C%i:BSWV STDEV,%f\r", channel + 1, amplitude);
	else
		Fmt (cmd, "C%i:BSWV AMP,%f\r", channel + 1, amplitude);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Offset
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainOffset (int channel, int wave, double offset)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (offset, -10.0, 10.0, -2) != 0)
        return gx1025_err;
        
	if (wave == 4)
		Fmt (cmd, "C%i:BSWV MEAN,%f\r", channel + 1, offset);
	else
		Fmt (cmd, "C%i:BSWV OFST,%f\r", channel + 1, offset);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Duty
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainSym (int channel, int wave, double symetry)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (wave, 1, 3, -2) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (symetry, 20.0, 80.0, -3) != 0)
        return gx1025_err;
        
	if (wave == 2)
		Fmt (cmd, "C%i:BSWV SYM,%f\r", channel + 1, symetry);
	else
		Fmt (cmd, "C%i:BSWV DUTY,%f\r", channel + 1, symetry);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of Phase
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainPhase (int channel, double phase)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (phase, 0.0, 360.0, -2) != 0)
        return gx1025_err;
        
    Fmt (cmd, "C%i:BSWV PHSE,%f\r", channel + 1, phase);
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Choose an arbitrary signal
//----------------------------------------------------------------
int _VI_FUNC gx1025_ArbSelect (int channel, int numStore)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (numStore, 2, 59, -2) != 0)
		return gx1025_err;
        
	Fmt (cmd, "C%i:ARWV INDEX,%i\r", channel + 1, numStore);
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}

//----------------------------------------------------------------
// Configuration of modulation
//----------------------------------------------------------------
int _VI_FUNC gx1025_setModulation (int channel, int func, double arg, double freqA, double freqB, int shape, int src)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	Fmt (cmd, "C%i:MDWV STATE,%s\r", channel + 1, onOff[func != 0]);
    gx1025_err = gx1025_send (cmd);

	if (func != OFF_MOD)
	{
		switch (func)
		{
			case AM:
				Fmt (cmd, "C%i:MDWV %s,DEPTH,%f,FRQ,%f,MDSP,%s,SRC,%s\r", 
								channel + 1, funcModul[func], arg, freqA, shapeFunc[shape], srceGen[src]);
				break;
			case DSB_AM:
				Fmt (cmd, "C%i:MDWV %s,FRQ,%f,MDSP,%s,SRC,%s\r", 
								channel + 1, funcModul[func], freqA, shapeFunc[shape], srceGen[src]);
				break;
			case ASK:
				Fmt (cmd, "C%i:MDWV %s,KFRQ,%f,SRC,%s\r", 
								channel + 1, funcModul[func], freqA, srceGen[src]);
				break;
			case FM:
				Fmt (cmd, "C%i:MDWV %s,FRQ,%f,DEVI,%f,MDSP,%s,SRC,%s\r", 
								channel + 1, funcModul[func], freqA, freqB, shapeFunc[shape], srceGen[src]);
				break;
			case FSK:
				Fmt (cmd, "C%i:MDWV %s,KFRQ,%f,HFRQ,%f,SRC,%s\r", 
								channel + 1, funcModul[func], freqA, freqB, srceGen[src]);
				break;
			case PM:
				Fmt (cmd, "C%i:MDWV %s,DEVI,%f,FRQ,%f,MDSP,%s,SRC,%s\r", 
								channel + 1, funcModul[func], arg, freqA, shapeFunc[shape], srceGen[src]);
				break;
		}
		gx1025_err = gx1025_send (cmd);
	}
	
	return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of sweep
//----------------------------------------------------------------
int _VI_FUNC gx1025_setSweep (int channel, double fStart, double fStop, double sTime, int src, int trig, int mode, int direc)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	Fmt (cmd, "C%i:SWWV STATE,ON,TIME,%f,STOP,%f,START,%f,TRSR,%s,TRMD,%s,SWMD,%s,DIR,%s\r", 
						channel + 1, sTime, fStop, fStart, srceGen[src], onOff[trig], swLaw[mode], sDirec[direc]);

	gx1025_err = gx1025_send (cmd);

	return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of burst
//----------------------------------------------------------------
int _VI_FUNC gx1025_setBurst (int channel, int typeB, double phase, int src, double sDelay, int Edge, int nbCycl, int Polar)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	Fmt (cmd, "C%i:BTWV STATE,ON,GATE_NCYC,%s,STPS,%f", channel + 1, typBurst[typeB], phase);
	if (typeB == 1)
		Fmt (cmd, "%s,PLRT,%s\r", cmd, polarity[Polar]);
	else
		Fmt (cmd, "%s,PRD,%f,TRSR,%s,TRMD,%s,TIME,%i\r", cmd, sDelay, srceGen[src], srceBurst[Edge], nbCycl);

	gx1025_err = gx1025_send (cmd);

	return gx1025_err;
}


//----------------------------------------------------------------
// Configuration of OUT
//----------------------------------------------------------------
int _VI_FUNC gx1025_MainOut (int channel, int output)
{
	char cmd[100];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (output, 0, 1, -2) != 0)
        return gx1025_err;
       
    Fmt (cmd, "C%i:OUTP %s\r", channel + 1, onOff[output]);
    
    gx1025_err = gx1025_send (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Runs the sweep
//----------------------------------------------------------------
int _VI_FUNC gx1025_trigSweep (int channel)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	Fmt (cmd, "C%i:SWWV MTRIG,ON\r", channel + 1);
    gx1025_err = gx1025_send (cmd);

	return gx1025_err;
}


//----------------------------------------------------------------
// Runs the burst
//----------------------------------------------------------------
int _VI_FUNC gx1025_trigBurst (int channel)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	Fmt (cmd, "C%i:BTWV MTRIG,ON\r", channel + 1);
    gx1025_err = gx1025_send (cmd);

	return gx1025_err;
}


//----------------------------------------------------------------
// Send an arbitrary signal in memory
//----------------------------------------------------------------
int _VI_FUNC gx1025_ArbStores (int numStore, char *nameStore, double freq, double emf, double dc, int tabTrace[])
{
	char *cmd;
	int cnt, i;
	short valS;

    if (gx1025_invalid_integer_range (numStore, 1, 10, -1) != 0)
		return gx1025_err;
	
	cmd = calloc (NB_PTS + 100, 2);
	Fmt (cmd, "WVDT M%i,WVNM,%s,TYPE,5,LENGTH,32KB,FREQ,%f,AMPL,%f,OFST,%f,PHASE,0.0,WAVEDATA,", 
							49 + numStore, nameStore, freq, emf, dc);
	cnt = NumFmtdBytes();
	i = 0;
	while (i < NB_PTS)
	{
		valS = (short)tabTrace[i];
		*(short*)(cmd + cnt) = valS & 0x3FFF;
		i ++;
		cnt += 2;
	}
	
	gx1025_err = viWrite (sessionUSB, cmd, cnt, &i);
	
	free (cmd);

    return gx1025_err;
}


//----------------------------------------------------------------
// Read an arbitrary signal saved in memory
//----------------------------------------------------------------
int _VI_FUNC gx1025_ArbRecall (int numStore, int *tabTrace)
{
	char *cmd;
	char buff[NB_PTS * 2 + 200];
	ViUInt32 retCnt;
	int cnt, i;
	short valS;

    if (gx1025_invalid_integer_range (numStore, 1, 10, -1) != 0)
		return gx1025_err;
	
	cmd = calloc (NB_PTS + 100, 2);
	Fmt (cmd, "WVDT M%i?\r", 49 + numStore);
    gx1025_err = gx1025_send (cmd);
	gx1025_err = viRead (sessionUSB, buff, NB_PTS * 2 + 200, &retCnt);
	
	if (retCnt != 0)
	{
		cnt = FindPattern (buff, 0, -1, "WAVEDATA", 1, 0);
		cnt += 9;
	
		i = 0;
		while (i < NB_PTS)
		{
			valS = *(short*)(buff + cnt);
			if (valS > 0x1FFF)
				tabTrace[i] = (long)(valS - 0x4000);
			else
				tabTrace[i] = (long)valS;
		
			i ++;
			cnt += 2;
		}
	}
    return gx1025_err;
}


//----------------------------------------------------------------
// Read the Setup 
//----------------------------------------------------------------
int _VI_FUNC gx1025_Setup (int channel, int *function, double *freq, double *ampl, double *offset, double *phase, double *duty, int *state)
{
	char cmd[100];
	char buff[500];
	ViUInt32 retCnt, indx;

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;

	// initialisation
	*function = 0;
	*freq = 0.0;
	*ampl = 0.0;
	*offset = 0.0;
	*phase = 0.0;
	*duty = 0.0;
	*state = 0;
	
	// demande de config
	memset (buff, 0, 500);
	Fmt (cmd, "C%i:BSWV?\r", channel + 1);
    gx1025_err = gx1025_send (cmd);
	gx1025_err = viRead (sessionUSB, buff, 200, &retCnt);
	
	// recherche du type de signal
	if ((indx = FindPattern (buff, 0, -1, "WVTP", 0, 0)) != -1)
	{
		if (FindPattern (buff, indx + 5, 7, "SINE", 0, 0) != -1)
			*function = 0;
		else if (FindPattern (buff, indx + 5, 7, "SQUARE", 0, 0) != -1)
			*function = 1;
		else if (FindPattern (buff, indx + 5, 7, "RAMP", 0, 0) != -1)
		{
			*function = 2;
			// recherche de la symetrie (duty)
			if ((indx = FindPattern (buff, 0, -1, "SYM", 0, 0)) != -1)
				Scan (buff + indx + 4, "%f", duty);
		}
		else if (FindPattern (buff, indx + 5, 7, "PULSE", 0, 0) != -1)
		{
			*function = 3;
			// recherche du delay
			if ((indx = FindPattern (buff, 0, -1, "DLY", 0, 0)) != -1)
				Scan (buff + indx + 4, "%f", phase);
		}
		else if (FindPattern (buff, indx + 5, 7, "NOISE", 0, 0) != -1)
		{
			*function = 4;
			indx = FindPattern (buff, 0, -1, "STDEV", 0, 0);
			Scan (buff + indx + 6, "%f", ampl);
			indx = FindPattern (buff, 0, -1, "MEAN", 0, 0);
			Scan (buff + indx + 5, "%f", offset);
		}
		else if (FindPattern (buff, indx + 5, 7, "ARB", 0, 0) != -1)
			*function = 5;
		else if (FindPattern (buff, indx + 5, 7, "DC", 0, 0) != -1)
			*function = 6;
	}
	
	// recherche de la frquence
	if ((indx = FindPattern (buff, 0, -1, "FRQ", 0, 0)) != -1)
		Scan (buff + indx + 4, "%f", freq);
	
	// recherche de l'amplitude
	if ((indx = FindPattern (buff, indx, -1, "AMP", 0, 0)) != -1)
		Scan (buff + indx + 4, "%f", ampl);

	// recherche de l'offset
	if ((indx = FindPattern (buff, 0, -1, "OFST", 0, 0)) != -1)
		Scan (buff + indx + 5, "%f", offset);

	// recherche de la phase
	if ((indx = FindPattern (buff, 0, -1, "PHSE", 0, 0)) != -1)
		Scan (buff + indx + 4, "%f", phase);

	// recherche du rapport cyclique
	if ((indx = FindPattern (buff, 0, -1, "DUTY", 0, 0)) != -1)
		Scan (buff + indx + 5, "%f", duty);

	// demande de l'etat de sortie : ON ou OFF
	memset (buff, 0, 500);
	Fmt (cmd, "C%i:OUTP?\r", channel + 1);
    gx1025_err = gx1025_send (cmd);
	gx1025_err = viRead (sessionUSB, buff, 200, &retCnt);
	if (FindPattern (buff, 0, -1, "ON", 0, 0) != -1)
		*state = 1;
	
    return gx1025_err;
}


//----------------------------------------------------------------
// Send a setup
//----------------------------------------------------------------
int _VI_FUNC gx1025_SendSetup (int channel, int func, double freq, double ampl, double offset, double phase, double duty, int state)
{
	char cmd[200];

    if (gx1025_invalid_integer_range (channel, 0, 1, -1) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (func, 0, 6, -2) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (freq, 1e-6, FreqMax, -3) != 0)
        return gx1025_err;
    if (func == 4)
	{
		if (gx1025_invalid_real_range (ampl, 0.0005, 20.0, -4) != 0)
	        return gx1025_err;
	}
	else
	{
	    if (gx1025_invalid_real_range (ampl, 0.004, 20.0, -4) != 0)
	        return gx1025_err;
	}
    if (gx1025_invalid_real_range (offset, -10.0, 10.0, -5) != 0)
        return gx1025_err;
     if (gx1025_invalid_real_range (phase, 0.0, 360.0, -6) != 0)
        return gx1025_err;
    if (gx1025_invalid_real_range (duty, 0.0, 100.0, -7) != 0)
        return gx1025_err;
    if (gx1025_invalid_integer_range (state, 0, 1, -8) != 0)
        return gx1025_err;

    Fmt (cmd, "C%i:BSWV WVTP,%s", channel + 1, function[func]);
    Fmt (cmd, "%s,FRQ,%f", cmd, freq);

	if (func == 4)
	{
		Fmt (cmd, "%s,STDEV,%f", cmd, ampl);
		Fmt (cmd, "%s,MEAN,%f", cmd, offset);
	}
	else
	{
		Fmt (cmd, "%s,AMP,%f", cmd, ampl);
		Fmt (cmd, "%s,OFST,%f", cmd, offset);
    }
    
	Fmt (cmd, "%s,PHSE,%f", cmd, phase);
	
	if (func == 1 || func == 3)
		Fmt (cmd, "%s,DUTY,%f\r", cmd, duty);
	else if (func == 2)
		Fmt (cmd, "%s,SYM,%f\r", cmd, duty);

    gx1025_err = gx1025_send (cmd);
	Delay (0.5);

	Fmt (cmd,"C%i:OUTP %s\r", channel + 1, onOff[state]);
    gx1025_err = gx1025_send (cmd);
	
    return gx1025_err;
}


//----------------------------------------------------------------
// Close the connection
//----------------------------------------------------------------
int _VI_FUNC gx1025_close (void)
{
	// fermeture de la session 
	viClose (sessionUSB);
	viClose (sessionCom);
	return 0;
}


//----------------------------------------------------------------
// Send command
//----------------------------------------------------------------
int gx1025_send (char *buff)
{
	int retCnt;
	ViStatus status;
	
	status = viWrite (sessionUSB, buff, strlen (buff), &retCnt);
	return status;
}


/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int gx1025_invalid_integer_range (int val, int min, int max, int err_code)
{
	if ((val < min) || (val > max))
    {
    	gx1025_err = err_code;
    	return -1;
    }
  	return 0;
}


/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int gx1025_invalid_real_range (double val, double min, double max, int err_code)
{
	if ((val < min) || (val > max))
    {
    	gx1025_err = err_code;
    	return -1;
    }
  	return 0;
}
